/**
 * The MIT License (MIT)
 *
 * Copyright (C) 2013 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.apidesign.demo.chat;

import net.java.html.json.ComputedProperty;
import net.java.html.json.Function;
import net.java.html.json.Model;
import net.java.html.json.OnReceive;
import net.java.html.json.Property;
import org.apidesign.demo.chat.shared.Message;
import org.apidesign.demo.chat.shared.Query;
import org.apidesign.demo.chat.shared.Reply;

/**
 *
 * @author Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 */
@Model(className = "ChatModel", targetId = "", properties = {
    @Property(name = "user", type = String.class),
    @Property(name = "comment", type = String.class),
    @Property(name = "url", type = String.class),
    @Property(name = "msgs", type = Message.class, array = true)
})
class ChatClient {
    @ComputedProperty
    static boolean sendEnabled(String user, String comment) {
        boolean res = user != null && comment != null && !user.isEmpty() && !comment.isEmpty();
        return res;
    }
    
    @Function
    static void submit(ChatModel m) {
        if (!sendEnabled(m.getUser(), m.getComment())) {
            return;
        }
        Message msg = new Message();
        msg.setUser(m.getUser());
        msg.setComment(m.getComment());
        m.socket(m.getUrl(), new Query().assignPost(msg));
    }
    
    static void cantChat(ChatModel model, Exception err) {
        model.getMsgs().add(new Message("System", err.getMessage(), 0));
    }
    
    
    @OnReceive(url = "{url}/chat", method = "WebSocket", data = Query.class, onError = "cantChat")
    static void socket(ChatModel m, Reply reply) {
        if (reply == null) {
            m.getMsgs().add(new Message("System", "Connected to server at " + m.getUrl() + "/chat!", 0));
            return;
        }
        m.getMsgs().clear();
        m.getMsgs().addAll(reply.getMessages());
    }
    
    public static void onPageLoad() {
        ChatModel chm = new ChatModel();
        chm.setUrl("ws://localhost:8080");
        Message m = new Message("System", "Waiting for messages from the server...", 0);
        chm.getMsgs().add(m);
        chm.applyBindings();
        chm.socket(chm.getUrl(), null);
    }
}
