/**
 * The MIT License (MIT)
 *
 * Copyright (C) 2013 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.apidesign.demo.chat.server;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import net.java.html.BrwsrCtx;
import net.java.html.json.Models;
import org.apidesign.demo.chat.shared.Message;
import org.apidesign.demo.chat.shared.Query;
import org.glassfish.grizzly.websockets.WebSocket;
import org.glassfish.grizzly.websockets.WebSocketApplication;

/** Server side of the chat application.*/
public final class ChatServerResource extends WebSocketApplication {
    private static final Logger LOG = Logger.getLogger(ChatServerResource.class.getName());
    private static final long started = System.currentTimeMillis() - 10;
    
    private List<Message> msgs = new ArrayList<>();
    {
        Message welcome = new Message();
        welcome.setUser("system");
        welcome.setComment("Welcome and enjoy!");
        welcome.setSince(10);
        msgs.add(welcome);
    }


/*
    private final Map<AsyncResponse, Long> awaiting = new IdentityHashMap<>();
    
    @Produces(MediaType.APPLICATION_JSON)
    @GET public synchronized void getResources(
        @QueryParam("since") @DefaultValue("0") long since,
        @Suspended AsyncResponse ar
    ) {
        Query q = new Query();
        for (Message m : msgs) {
            if (m.getSince()>= since) {
                q.getMessages().add(m);
            }
        }
        if (!q.getMessages().isEmpty()) {
            ar.resume(q);
        } else {
            awaiting.put(ar, since);
        }
    }
    
    private void handleAwaiting(long newest) {
        assert Thread.holdsLock(this);
        AGAIN: for (;;) {
            for (Map.Entry<AsyncResponse, Long> entry : awaiting.entrySet()) {
                AsyncResponse ar = entry.getKey();
                Long since = entry.getValue();
                if (since <= newest) {
                    awaiting.remove(ar);
                    getResources(since, ar);
                    continue AGAIN;
                }
            }
            return;
        }
    }
    
    @POST @Consumes(value = MediaType.APPLICATION_JSON)
    public synchronized Message publish(Message msg) {
        msg.setSince(System.currentTimeMillis() - started);
        msgs.add(msg);
        handleAwaiting(msg.getSince());
        return msg;
    }
*/

    @Override
    public void onMessage(WebSocket socket, String text) {
        try {
            ByteArrayInputStream is = new ByteArrayInputStream(text.getBytes("UTF-8"));
            Query q = Models.parse(BrwsrCtx.findDefault(ChatServerResource.class), Query.class, is);
            super.onMessage(socket, text);
        } catch (IOException ex) {
            throw new IllegalStateException(ex);
        }
    }
}
